@@ -22,6 +22,27 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-esdsink
+ * @see_also: #GstAlsaSink, #GstAutoAudioSink
+ *
+ * This element outputs sound to an already-running Enlightened Sound Daemon
+ * (ESound Daemon, esd). Note that a sound daemon will never be auto-spawned
+ * through this element (regardless of the system configuration), since this
+ * is actively prevented by the element. If you must use esd, you need to
+ * make sure it is started automatically with your session or otherwise.
+ *
+ * TODO: insert some comments about how sucky esd is and that all the cool
+ * kids use pulseaudio or whatever these days.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch -v filesrc location=foo.ogg ! decodebin ! audioconvert ! audioresample ! esdsink
+ * ]| play an Ogg/Vorbis audio file via esd
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -58,7 +79,7 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
-        "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", "
+        "endianness = (int) BYTE_ORDER, "
         "signed = (boolean) TRUE, "
         "width = (int) 16, "
         "depth = (int) 16, "
@@ -71,9 +92,6 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
         "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]")
     );
 
-static void gst_esdsink_base_init (gpointer g_class);
-static void gst_esdsink_class_init (GstEsdSinkClass * klass);
-static void gst_esdsink_init (GstEsdSink * esdsink);
 static void gst_esdsink_finalize (GObject * object);
 
 static GstCaps *gst_esdsink_getcaps (GstBaseSink * bsink);
@@ -93,32 +111,7 @@ static void gst_esdsink_set_property (GObject * object, guint prop_id,
 static void gst_esdsink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static GstElementClass *parent_class = NULL;
-
-GType
-gst_esdsink_get_type (void)
-{
-  static GType esdsink_type = 0;
-
-  if (!esdsink_type) {
-    static const GTypeInfo esdsink_info = {
-      sizeof (GstEsdSinkClass),
-      gst_esdsink_base_init,
-      NULL,
-      (GClassInitFunc) gst_esdsink_class_init,
-      NULL,
-      NULL,
-      sizeof (GstEsdSink),
-      0,
-      (GInstanceInitFunc) gst_esdsink_init,
-    };
-
-    esdsink_type =
-        g_type_register_static (GST_TYPE_AUDIO_SINK, "GstEsdSink",
-        &esdsink_info, 0);
-  }
-  return esdsink_type;
-}
+GST_BOILERPLATE (GstEsdSink, gst_esdsink, GstAudioSink, GST_TYPE_AUDIO_SINK);
 
 static void
 gst_esdsink_base_init (gpointer g_class)
@@ -135,12 +128,10 @@ gst_esdsink_class_init (GstEsdSinkClass * klass)
 {
   GObjectClass *gobject_class;
   GstBaseSinkClass *gstbasesink_class;
-  GstBaseAudioSinkClass *gstbaseaudiosink_class;
   GstAudioSinkClass *gstaudiosink_class;
 
   gobject_class = (GObjectClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
-  gstbaseaudiosink_class = (GstBaseAudioSinkClass *) klass;
   gstaudiosink_class = (GstAudioSinkClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -167,7 +158,7 @@ gst_esdsink_class_init (GstEsdSinkClass * klass)
 }
 
 static void
-gst_esdsink_init (GstEsdSink * esdsink)
+gst_esdsink_init (GstEsdSink * esdsink, GstEsdSinkClass * klass)
 {
   esdsink->fd = -1;
   esdsink->ctrl_fd = -1;
@@ -179,6 +170,7 @@ gst_esdsink_finalize (GObject * object)
 {
   GstEsdSink *esdsink = GST_ESDSINK (object);
 
+  gst_caps_replace (&esdsink->cur_caps, NULL);
   g_free (esdsink->host);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -188,60 +180,73 @@ static GstCaps *
 gst_esdsink_getcaps (GstBaseSink * bsink)
 {
   GstEsdSink *esdsink;
-  GstPadTemplate *pad_template;
-  GstCaps *caps = NULL;
-  gint i;
-  esd_server_info_t *server_info;
 
   esdsink = GST_ESDSINK (bsink);
 
-  GST_DEBUG_OBJECT (esdsink, "getcaps called");
-
-  pad_template = gst_static_pad_template_get (&sink_factory);
-  caps = gst_caps_copy (gst_pad_template_get_caps (pad_template));
-
   /* no fd, we're done with the template caps */
-  if (esdsink->ctrl_fd < 0)
-    goto done;
-
-  /* get server info */
-  server_info = esd_get_server_info (esdsink->ctrl_fd);
-  if (!server_info)
-    goto no_info;
-
-  GST_DEBUG_OBJECT (esdsink, "got server info rate: %i", server_info->rate);
-
-  for (i = 0; i < caps->structs->len; i++) {
-    GstStructure *s;
-
-    s = gst_caps_get_structure (caps, i);
-    gst_structure_set (s, "rate", G_TYPE_INT, server_info->rate, NULL);
-  }
-  esd_free_server_info (server_info);
-
-done:
-  return caps;
-
-  /* ERRORS */
-no_info:
-  {
-    GST_WARNING_OBJECT (esdsink, "couldn't get server info!");
-    gst_caps_unref (caps);
+  if (esdsink->ctrl_fd < 0 || esdsink->cur_caps == NULL) {
+    GST_LOG_OBJECT (esdsink, "getcaps called, returning template caps");
     return NULL;
   }
+
+  GST_LOG_OBJECT (esdsink, "returning %" GST_PTR_FORMAT, esdsink->cur_caps);
+
+  return gst_caps_ref (esdsink->cur_caps);
 }
 
 static gboolean
 gst_esdsink_open (GstAudioSink * asink)
 {
-  GstEsdSink *esdsink = GST_ESDSINK (asink);
+  esd_server_info_t *server_info;
+  GstPadTemplate *pad_template;
+  GstEsdSink *esdsink;
+  gchar *saved_env;
+  gint i;
+
+  esdsink = GST_ESDSINK (asink);
 
   GST_DEBUG_OBJECT (esdsink, "open");
 
+  /* ensure libesd doesn't auto-spawn a sound daemon if none is running yet */
+  saved_env = g_strdup (g_getenv ("ESD_NO_SPAWN"));
+  g_setenv ("ESD_NO_SPAWN", "1", TRUE);
+
+  /* now try to connect to any existing/running sound daemons */
   esdsink->ctrl_fd = esd_open_sound (esdsink->host);
+
+  /* and restore the previous state */
+  if (saved_env != NULL) {
+    g_setenv ("ESD_NO_SPAWN", saved_env, TRUE);
+  } else {
+    g_unsetenv ("ESD_NO_SPAWN");
+  }
+  g_free (saved_env);
+
   if (esdsink->ctrl_fd < 0)
     goto couldnt_connect;
 
+  /* get server info */
+  server_info = esd_get_server_info (esdsink->ctrl_fd);
+  if (!server_info)
+    goto no_server_info;
+
+  GST_INFO_OBJECT (esdsink, "got server info rate: %i", server_info->rate);
+
+  pad_template = gst_static_pad_template_get (&sink_factory);
+  esdsink->cur_caps = gst_caps_copy (gst_pad_template_get_caps (pad_template));
+  gst_object_unref (pad_template);
+
+  for (i = 0; i < esdsink->cur_caps->structs->len; i++) {
+    GstStructure *s;
+
+    s = gst_caps_get_structure (esdsink->cur_caps, i);
+    gst_structure_set (s, "rate", G_TYPE_INT, server_info->rate, NULL);
+  }
+
+  esd_free_server_info (server_info);
+
+  GST_INFO_OBJECT (esdsink, "server caps: %" GST_PTR_FORMAT, esdsink->cur_caps);
+
   return TRUE;
 
   /* ERRORS */
@@ -252,6 +257,13 @@ couldnt_connect:
         ("can't open connection to esound server"));
     return FALSE;
   }
+no_server_info:
+  {
+    GST_ELEMENT_ERROR (esdsink, RESOURCE, OPEN_WRITE,
+        (_("Failed to query sound server capabilities")),
+        ("couldn't get server info!"));
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -261,6 +273,7 @@ gst_esdsink_close (GstAudioSink * asink)
 
   GST_DEBUG_OBJECT (esdsink, "close");
 
+  gst_caps_replace (&esdsink->cur_caps, NULL);
   esd_close (esdsink->ctrl_fd);
   esdsink->ctrl_fd = -1;
 
@@ -316,6 +329,9 @@ gst_esdsink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
 
   spec->segsize = ESD_BUF_SIZE;
   spec->segtotal = (ESD_MAX_WRITE_SIZE / spec->segsize);
+
+  /* FIXME: this is wrong for signed ints (and the
+   * audioringbuffers should do it for us anyway) */
   spec->silence_sample[0] = 0;
   spec->silence_sample[1] = 0;
   spec->silence_sample[2] = 0;
@@ -382,7 +398,7 @@ gst_esdsink_write (GstAudioSink * asink, gpointer data, guint length)
       goto write_error;
 
     to_write -= done;
-    data += done;
+    data = (char *) data + done;
   }
   return length;
 
@@ -391,7 +407,7 @@ write_error:
   {
     GST_ELEMENT_ERROR (esdsink, RESOURCE, WRITE,
         ("Failed to write data to the esound daemon"), GST_ERROR_SYSTEM);
-    return 0;
+    return -1;
   }
 }
 
@@ -403,9 +419,14 @@ gst_esdsink_delay (GstAudioSink * asink)
 
   latency = esd_get_latency (esdsink->ctrl_fd);
 
+  if (latency == (guint) - 1) {
+    GST_WARNING_OBJECT (asink, "couldn't get latency");
+    return 0;
+  }
+
   /* latency is measured in samples at a rate of 44100, this 
    * cannot overflow. */
-  latency = latency * 44100LL / esdsink->rate;
+  latency = latency * G_GINT64_CONSTANT (44100) / esdsink->rate;
 
   GST_DEBUG_OBJECT (asink, "got latency: %u", latency);
 
@@ -449,13 +470,3 @@ gst_esdsink_get_property (GObject * object, guint prop_id, GValue * value,
       break;
   }
 }
-
-gboolean
-gst_esdsink_factory_init (GstPlugin * plugin)
-{
-  if (!gst_element_register (plugin, "esdsink", GST_RANK_NONE,
-          GST_TYPE_ESDSINK))
-    return FALSE;
-
-  return TRUE;
-}
