@@ -52,7 +52,6 @@
 
 #include "gstapedemux.h"
 
-#include <stdio.h>
 #include <string.h>
 
 #define APE_VERSION_MAJOR(ver)  ((ver)/1000)
@@ -60,6 +59,13 @@
 GST_DEBUG_CATEGORY_STATIC (apedemux_debug);
 #define GST_CAT_DEFAULT (apedemux_debug)
 
+static const GstElementDetails gst_ape_demux_details =
+GST_ELEMENT_DETAILS ("APE tag demuxer",
+    "Codec/Demuxer/Metadata",
+    "Read and output APE tags while demuxing the contents",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>, "
+    "Tim-Philipp Müller <tim centricular net>");
+
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -79,12 +85,10 @@ gst_ape_demux_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 
-  gst_element_class_set_details_simple (element_class, "APE tag demuxer",
-      "Codec/Demuxer/Metadata",
-      "Read and output APE tags while demuxing the contents",
-      "Tim-Philipp Müller <tim centricular net>");
+  gst_element_class_set_details (element_class, &gst_ape_demux_details);
 
   GST_DEBUG_CATEGORY_INIT (apedemux_debug, "apedemux", 0,
       "GStreamer APE tag demuxer");
@@ -196,16 +200,10 @@ ape_demux_parse_tags (const guint8 * data, gint size)
     if (size - n < len)
       break;
 
-    /* If the tag is empty, skip to the next one */
-    if (len == 0)
-      goto next_tag;
-
     /* read */
     tag = g_strndup ((gchar *) data + 8, n - 9);
     val = g_strndup ((gchar *) data + n, len);
 
-    GST_LOG ("tag [%s], val[%s]", tag, val);
-
     /* special-case 'media' tag, could be e.g. "CD 1/2" */
     if (g_ascii_strcasecmp (tag, "media") == 0) {
       gchar *sp, *sp2;
@@ -319,7 +317,6 @@ ape_demux_parse_tags (const guint8 * data, gint size)
     g_free (val);
 
     /* move data pointer */
-  next_tag:
     size -= len + n;
     data += len + n;
   }
