@@ -468,19 +468,17 @@ gst_flv_demux_create_index (GstFLVDemux * demux)
   size_t tag_size;
   guint64 old_offset;
   GstBuffer *buffer;
-  GstFlowReturn ret;
 
-  if (!gst_pad_query_peer_duration (demux->sinkpad, &fmt, &size) ||
-      fmt != GST_FORMAT_BYTES)
+  if (G_UNLIKELY (!gst_pad_query_peer_duration (demux->sinkpad, &fmt, &size) ||
+          fmt != GST_FORMAT_BYTES))
     return;
 
   old_offset = demux->offset;
 
-  while ((ret =
-          gst_flv_demux_pull_range (demux, demux->sinkpad, demux->offset, 12,
-              &buffer)) == GST_FLOW_OK) {
-    if (gst_flv_parse_tag_timestamp (demux, buffer,
-            &tag_size) == GST_CLOCK_TIME_NONE) {
+  while (gst_flv_demux_pull_range (demux, demux->sinkpad, demux->offset, 12,
+          &buffer) == GST_FLOW_OK) {
+    if (G_UNLIKELY (gst_flv_parse_tag_timestamp (demux, buffer,
+                &tag_size) == GST_CLOCK_TIME_NONE)) {
       gst_buffer_unref (buffer);
       break;
     }
@@ -671,6 +669,7 @@ gst_flv_demux_handle_seek_push (GstFLVDemux * demux, GstEvent * event)
     goto wrong_format;
 
   flush = !!(flags & GST_SEEK_FLAG_FLUSH);
+  /* FIXME : the keyframe flag is never used ! */
   keyframe = !!(flags & GST_SEEK_FLAG_KEY_UNIT);
 
   /* Work on a copy until we are sure the seek succeeded. */
@@ -755,6 +754,7 @@ gst_flv_demux_handle_seek_pull (GstFLVDemux * demux, GstEvent * event)
     goto wrong_format;
 
   flush = !!(flags & GST_SEEK_FLAG_FLUSH);
+  /* FIXME : the keyframe flag is never used */
   keyframe = !!(flags & GST_SEEK_FLAG_KEY_UNIT);
 
   if (flush) {
@@ -1077,6 +1077,32 @@ gst_flv_demux_query (GstPad * pad, GstQuery * query)
       break;
     }
 
+    case GST_QUERY_SEEKING:{
+      GstFormat fmt;
+
+      gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
+      res = TRUE;
+      if (fmt != GST_FORMAT_TIME || !demux->index) {
+        gst_query_set_seeking (query, fmt, FALSE, -1, -1);
+      } else if (demux->random_access) {
+        gst_query_set_seeking (query, GST_FORMAT_TIME, TRUE, 0,
+            demux->duration);
+      } else {
+        GstQuery *peerquery = gst_query_new_seeking (GST_FORMAT_BYTES);
+        gboolean seekable = gst_pad_peer_query (demux->sinkpad, peerquery);
+
+        if (seekable)
+          gst_query_parse_seeking (peerquery, NULL, &seekable, NULL, NULL);
+        gst_query_unref (peerquery);
+
+        if (seekable)
+          gst_query_set_seeking (query, GST_FORMAT_TIME, seekable, 0,
+              demux->duration);
+        else
+          gst_query_set_seeking (query, GST_FORMAT_TIME, FALSE, -1, -1);
+      }
+      break;
+    }
     case GST_QUERY_LATENCY:
     default:
     {
