@@ -44,10 +44,6 @@
 #include "config.h"
 #endif
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include <string.h>
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/rtp/gstrtcpbuffer.h>
@@ -87,8 +83,15 @@ GST_STATIC_PAD_TEMPLATE ("rtcp_src_%d",
     GST_STATIC_CAPS ("application/x-rtcp")
     );
 
-#define GST_PAD_LOCK(obj)   (g_static_rec_mutex_lock (&(obj)->padlock))
-#define GST_PAD_UNLOCK(obj) (g_static_rec_mutex_unlock (&(obj)->padlock))
+static GstElementDetails gst_rtp_ssrc_demux_details = {
+  "RTP SSRC Demux",
+  "Demux/Network/RTP",
+  "Splits RTP streams based on the SSRC",
+  "Wim Taymans <wim.taymans@gmail.com>"
+};
+
+#define GST_PAD_LOCK(obj)   (g_mutex_lock ((obj)->padlock))
+#define GST_PAD_UNLOCK(obj) (g_mutex_unlock ((obj)->padlock))
 
 /* signals */
 enum
@@ -122,13 +125,10 @@ static GstFlowReturn gst_rtp_ssrc_demux_rtcp_chain (GstPad * pad,
     GstBuffer * buf);
 static gboolean gst_rtp_ssrc_demux_rtcp_sink_event (GstPad * pad,
     GstEvent * event);
-static GstIterator *gst_rtp_ssrc_demux_iterate_internal_links_sink (GstPad *
-    pad);
 
 /* srcpad stuff */
 static gboolean gst_rtp_ssrc_demux_src_event (GstPad * pad, GstEvent * event);
-static GstIterator *gst_rtp_ssrc_demux_iterate_internal_links_src (GstPad *
-    pad);
+static GList *gst_rtp_ssrc_demux_internal_links (GstPad * pad);
 static gboolean gst_rtp_ssrc_demux_src_query (GstPad * pad, GstQuery * query);
 
 static guint gst_rtp_ssrc_demux_signals[LAST_SIGNAL] = { 0 };
@@ -162,7 +162,8 @@ find_demux_pad_for_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc)
 
 /* with PAD_LOCK */
 static GstRtpSsrcDemuxPad *
-find_or_create_demux_pad_for_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc)
+create_demux_pad_for_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc,
+    GstClockTime timestamp)
 {
   GstPad *rtp_pad, *rtcp_pad;
   GstElementClass *klass;
@@ -172,11 +173,6 @@ find_or_create_demux_pad_for_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc)
 
   GST_DEBUG_OBJECT (demux, "creating pad for SSRC %08x", ssrc);
 
-  demuxpad = find_demux_pad_for_ssrc (demux, ssrc);
-  if (demuxpad != NULL) {
-    return demuxpad;
-  }
-
   klass = GST_ELEMENT_GET_CLASS (demux);
   templ = gst_element_class_get_pad_template (klass, "src_%d");
   padname = g_strdup_printf ("src_%d", ssrc);
@@ -188,12 +184,20 @@ find_or_create_demux_pad_for_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc)
   rtcp_pad = gst_pad_new_from_template (templ, padname);
   g_free (padname);
 
+  /* we use the first timestamp received to calculate the difference between
+   * timestamps on all streams */
+  GST_DEBUG_OBJECT (demux, "SSRC %08x, first timestamp %" GST_TIME_FORMAT,
+      ssrc, GST_TIME_ARGS (timestamp));
+
   /* wrap in structure and add to list */
   demuxpad = g_new0 (GstRtpSsrcDemuxPad, 1);
   demuxpad->ssrc = ssrc;
   demuxpad->rtp_pad = rtp_pad;
   demuxpad->rtcp_pad = rtcp_pad;
 
+  GST_DEBUG_OBJECT (demux, "first timestamp %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (timestamp));
+
   gst_pad_set_element_private (rtp_pad, demuxpad);
   gst_pad_set_element_private (rtcp_pad, demuxpad);
 
@@ -207,13 +211,12 @@ find_or_create_demux_pad_for_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc)
 
   gst_pad_set_event_function (rtp_pad, gst_rtp_ssrc_demux_src_event);
   gst_pad_set_query_function (rtp_pad, gst_rtp_ssrc_demux_src_query);
-  gst_pad_set_iterate_internal_links_function (rtp_pad,
-      gst_rtp_ssrc_demux_iterate_internal_links_src);
+  gst_pad_set_internal_link_function (rtp_pad,
+      gst_rtp_ssrc_demux_internal_links);
   gst_pad_set_active (rtp_pad, TRUE);
 
-  gst_pad_set_event_function (rtcp_pad, gst_rtp_ssrc_demux_src_event);
-  gst_pad_set_iterate_internal_links_function (rtcp_pad,
-      gst_rtp_ssrc_demux_iterate_internal_links_src);
+  gst_pad_set_internal_link_function (rtcp_pad,
+      gst_rtp_ssrc_demux_internal_links);
   gst_pad_set_active (rtcp_pad, TRUE);
 
   gst_element_add_pad (GST_ELEMENT_CAST (demux), rtp_pad);
@@ -230,19 +233,16 @@ gst_rtp_ssrc_demux_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_klass = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (gstelement_klass,
-      &rtp_ssrc_demux_sink_template);
-  gst_element_class_add_static_pad_template (gstelement_klass,
-      &rtp_ssrc_demux_rtcp_sink_template);
-  gst_element_class_add_static_pad_template (gstelement_klass,
-      &rtp_ssrc_demux_src_template);
-  gst_element_class_add_static_pad_template (gstelement_klass,
-      &rtp_ssrc_demux_rtcp_src_template);
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&rtp_ssrc_demux_sink_template));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&rtp_ssrc_demux_rtcp_sink_template));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&rtp_ssrc_demux_src_template));
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&rtp_ssrc_demux_rtcp_src_template));
 
-  gst_element_class_set_details_simple (gstelement_klass, "RTP SSRC Demux",
-      "Demux/Network/RTP",
-      "Splits RTP streams based on the SSRC",
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (gstelement_klass, &gst_rtp_ssrc_demux_details);
 }
 
 static void
@@ -256,8 +256,8 @@ gst_rtp_ssrc_demux_class_init (GstRtpSsrcDemuxClass * klass)
   gstelement_klass = (GstElementClass *) klass;
   gstrtpssrcdemux_klass = (GstRtpSsrcDemuxClass *) klass;
 
-  gobject_klass->dispose = gst_rtp_ssrc_demux_dispose;
-  gobject_klass->finalize = gst_rtp_ssrc_demux_finalize;
+  gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_rtp_ssrc_demux_dispose);
+  gobject_klass->finalize = GST_DEBUG_FUNCPTR (gst_rtp_ssrc_demux_finalize);
 
   /**
    * GstRtpSsrcDemux::new-ssrc-pad:
@@ -322,8 +322,6 @@ gst_rtp_ssrc_demux_init (GstRtpSsrcDemux * demux,
           "sink"), "sink");
   gst_pad_set_chain_function (demux->rtp_sink, gst_rtp_ssrc_demux_chain);
   gst_pad_set_event_function (demux->rtp_sink, gst_rtp_ssrc_demux_sink_event);
-  gst_pad_set_iterate_internal_links_function (demux->rtp_sink,
-      gst_rtp_ssrc_demux_iterate_internal_links_sink);
   gst_element_add_pad (GST_ELEMENT_CAST (demux), demux->rtp_sink);
 
   demux->rtcp_sink =
@@ -332,11 +330,9 @@ gst_rtp_ssrc_demux_init (GstRtpSsrcDemux * demux,
   gst_pad_set_chain_function (demux->rtcp_sink, gst_rtp_ssrc_demux_rtcp_chain);
   gst_pad_set_event_function (demux->rtcp_sink,
       gst_rtp_ssrc_demux_rtcp_sink_event);
-  gst_pad_set_iterate_internal_links_function (demux->rtcp_sink,
-      gst_rtp_ssrc_demux_iterate_internal_links_sink);
   gst_element_add_pad (GST_ELEMENT_CAST (demux), demux->rtcp_sink);
 
-  g_static_rec_mutex_init (&demux->padlock);
+  demux->padlock = g_mutex_new ();
 
   gst_segment_init (&demux->segment, GST_FORMAT_UNDEFINED);
 }
@@ -378,7 +374,7 @@ gst_rtp_ssrc_demux_finalize (GObject * object)
   GstRtpSsrcDemux *demux;
 
   demux = GST_RTP_SSRC_DEMUX (object);
-  g_static_rec_mutex_free (&demux->padlock);
+  g_mutex_free (demux->padlock);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -390,10 +386,8 @@ gst_rtp_ssrc_demux_clear_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc)
 
   GST_PAD_LOCK (demux);
   dpad = find_demux_pad_for_ssrc (demux, ssrc);
-  if (dpad == NULL) {
-    GST_PAD_UNLOCK (demux);
+  if (dpad != NULL)
     goto unknown_pad;
-  }
 
   GST_DEBUG_OBJECT (demux, "clearing pad for SSRC %08x", ssrc);
 
@@ -417,7 +411,7 @@ gst_rtp_ssrc_demux_clear_ssrc (GstRtpSsrcDemux * demux, guint32 ssrc)
   /* ERRORS */
 unknown_pad:
   {
-    GST_WARNING_OBJECT (demux, "unknown SSRC %08x", ssrc);
+    g_warning ("unknown SSRC %08x", ssrc);
     return;
   }
 }
@@ -429,10 +423,6 @@ gst_rtp_ssrc_demux_sink_event (GstPad * pad, GstEvent * event)
   gboolean res = FALSE;
 
   demux = GST_RTP_SSRC_DEMUX (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (demux == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_STOP:
@@ -441,28 +431,16 @@ gst_rtp_ssrc_demux_sink_event (GstPad * pad, GstEvent * event)
     default:
     {
       GSList *walk;
-      GSList *pads = NULL;
 
       res = TRUE;
-      /* need local snapshot of pads;
-       * should not push downstream while holding lock as that might deadlock
-       * with stuff traveling upstream tyring to get this lock while holding
-       * other (stream)lock */
       GST_PAD_LOCK (demux);
       for (walk = demux->srcpads; walk; walk = g_slist_next (walk)) {
         GstRtpSsrcDemuxPad *pad = (GstRtpSsrcDemuxPad *) walk->data;
 
-        pads = g_slist_prepend (pads, gst_object_ref (pad->rtp_pad));
+        gst_event_ref (event);
+        res &= gst_pad_push_event (pad->rtp_pad, event);
       }
       GST_PAD_UNLOCK (demux);
-      for (walk = pads; walk; walk = g_slist_next (walk)) {
-        GstPad *pad = (GstPad *) walk->data;
-
-        gst_event_ref (event);
-        res &= gst_pad_push_event (pad, event);
-        gst_object_unref (pad);
-      }
-      g_slist_free (pads);
       gst_event_unref (event);
       break;
     }
@@ -485,24 +463,16 @@ gst_rtp_ssrc_demux_rtcp_sink_event (GstPad * pad, GstEvent * event)
     default:
     {
       GSList *walk;
-      GSList *pads = NULL;
 
       res = TRUE;
       GST_PAD_LOCK (demux);
       for (walk = demux->srcpads; walk; walk = g_slist_next (walk)) {
         GstRtpSsrcDemuxPad *pad = (GstRtpSsrcDemuxPad *) walk->data;
 
-        pads = g_slist_prepend (pads, gst_object_ref (pad->rtcp_pad));
+        gst_event_ref (event);
+        res &= gst_pad_push_event (pad->rtcp_pad, event);
       }
       GST_PAD_UNLOCK (demux);
-      for (walk = pads; walk; walk = g_slist_next (walk)) {
-        GstPad *pad = (GstPad *) walk->data;
-
-        gst_event_ref (event);
-        res &= gst_pad_push_event (pad, event);
-        gst_object_unref (pad);
-      }
-      g_slist_free (pads);
       gst_event_unref (event);
       break;
     }
@@ -518,7 +488,6 @@ gst_rtp_ssrc_demux_chain (GstPad * pad, GstBuffer * buf)
   GstRtpSsrcDemux *demux;
   guint32 ssrc;
   GstRtpSsrcDemuxPad *dpad;
-  GstPad *srcpad;
 
   demux = GST_RTP_SSRC_DEMUX (GST_OBJECT_PARENT (pad));
 
@@ -530,18 +499,17 @@ gst_rtp_ssrc_demux_chain (GstPad * pad, GstBuffer * buf)
   GST_DEBUG_OBJECT (demux, "received buffer of SSRC %08x", ssrc);
 
   GST_PAD_LOCK (demux);
-  dpad = find_or_create_demux_pad_for_ssrc (demux, ssrc);
+  dpad = find_demux_pad_for_ssrc (demux, ssrc);
   if (dpad == NULL) {
-    GST_PAD_UNLOCK (demux);
-    goto create_failed;
+    if (!(dpad =
+            create_demux_pad_for_ssrc (demux, ssrc,
+                GST_BUFFER_TIMESTAMP (buf))))
+      goto create_failed;
   }
-  srcpad = gst_object_ref (dpad->rtp_pad);
   GST_PAD_UNLOCK (demux);
 
   /* push to srcpad */
-  ret = gst_pad_push (srcpad, buf);
-
-  gst_object_unref (srcpad);
+  ret = gst_pad_push (dpad->rtp_pad, buf);
 
   return ret;
 
@@ -558,6 +526,7 @@ create_failed:
   {
     GST_ELEMENT_ERROR (demux, STREAM, DECODE, (NULL),
         ("Could not create new pad"));
+    GST_PAD_UNLOCK (demux);
     gst_buffer_unref (buf);
     return GST_FLOW_ERROR;
   }
@@ -571,7 +540,6 @@ gst_rtp_ssrc_demux_rtcp_chain (GstPad * pad, GstBuffer * buf)
   guint32 ssrc;
   GstRtpSsrcDemuxPad *dpad;
   GstRTCPPacket packet;
-  GstPad *srcpad;
 
   demux = GST_RTP_SSRC_DEMUX (GST_OBJECT_PARENT (pad));
 
@@ -595,18 +563,16 @@ gst_rtp_ssrc_demux_rtcp_chain (GstPad * pad, GstBuffer * buf)
   GST_DEBUG_OBJECT (demux, "received RTCP of SSRC %08x", ssrc);
 
   GST_PAD_LOCK (demux);
-  dpad = find_or_create_demux_pad_for_ssrc (demux, ssrc);
+  dpad = find_demux_pad_for_ssrc (demux, ssrc);
   if (dpad == NULL) {
-    GST_PAD_UNLOCK (demux);
-    goto create_failed;
+    GST_DEBUG_OBJECT (demux, "creating pad for SSRC %08x", ssrc);
+    if (!(dpad = create_demux_pad_for_ssrc (demux, ssrc, -1)))
+      goto create_failed;
   }
-  srcpad = gst_object_ref (dpad->rtcp_pad);
   GST_PAD_UNLOCK (demux);
 
   /* push to srcpad */
-  ret = gst_pad_push (srcpad, buf);
-
-  gst_object_unref (srcpad);
+  ret = gst_pad_push (dpad->rtcp_pad, buf);
 
   return ret;
 
@@ -629,6 +595,7 @@ create_failed:
   {
     GST_ELEMENT_ERROR (demux, STREAM, DECODE, (NULL),
         ("Could not create new pad"));
+    GST_PAD_UNLOCK (demux);
     gst_buffer_unref (buf);
     return GST_FLOW_ERROR;
   }
@@ -638,117 +605,51 @@ static gboolean
 gst_rtp_ssrc_demux_src_event (GstPad * pad, GstEvent * event)
 {
   GstRtpSsrcDemux *demux;
-  const GstStructure *s;
+  gboolean res = FALSE;
 
   demux = GST_RTP_SSRC_DEMUX (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_CUSTOM_UPSTREAM:
-    case GST_EVENT_CUSTOM_BOTH:
-    case GST_EVENT_CUSTOM_BOTH_OOB:
-      s = gst_event_get_structure (event);
-      if (s && !gst_structure_has_field (s, "ssrc")) {
-        GSList *walk;
-
-        for (walk = demux->srcpads; walk; walk = g_slist_next (walk)) {
-          GstRtpSsrcDemuxPad *dpad = (GstRtpSsrcDemuxPad *) walk->data;
-
-          if (dpad->rtp_pad == pad || dpad->rtcp_pad == pad) {
-            event =
-                GST_EVENT_CAST (gst_mini_object_make_writable
-                (GST_MINI_OBJECT_CAST (event)));
-            gst_structure_set (event->structure, "ssrc", G_TYPE_UINT,
-                dpad->ssrc, NULL);
-            break;
-          }
-        }
-      }
-      break;
+    case GST_EVENT_SEEK:
     default:
+      res = gst_pad_event_default (pad, event);
       break;
   }
-
   gst_object_unref (demux);
-
-  return gst_pad_event_default (pad, event);
-}
-
-static GstIterator *
-gst_rtp_ssrc_demux_iterate_internal_links_src (GstPad * pad)
-{
-  GstRtpSsrcDemux *demux;
-  GstPad *otherpad = NULL;
-  GstIterator *it = NULL;
-  GSList *current;
-
-  demux = GST_RTP_SSRC_DEMUX (gst_pad_get_parent (pad));
-
-  if (!demux)
-    return NULL;
-
-  GST_PAD_LOCK (demux);
-  for (current = demux->srcpads; current; current = g_slist_next (current)) {
-    GstRtpSsrcDemuxPad *dpad = (GstRtpSsrcDemuxPad *) current->data;
-
-    if (pad == dpad->rtp_pad) {
-      otherpad = demux->rtp_sink;
-      break;
-    } else if (pad == dpad->rtcp_pad) {
-      otherpad = demux->rtcp_sink;
-      break;
-    }
-  }
-  it = gst_iterator_new_single (GST_TYPE_PAD, otherpad,
-      (GstCopyFunction) gst_object_ref, (GFreeFunc) gst_object_unref);
-  GST_PAD_UNLOCK (demux);
-
-  gst_object_unref (demux);
-  return it;
-}
-
-/* Should return 0 for elements to be included */
-static gint
-src_pad_compare_func (gconstpointer a, gconstpointer b)
-{
-  GstPad *pad = GST_PAD (a);
-  const gchar *prefix = b;
-  gint res = 1;
-
-  GST_OBJECT_LOCK (pad);
-  res = !GST_PAD_NAME (pad) || g_str_has_prefix (GST_PAD_NAME (pad), prefix);
-  GST_OBJECT_UNLOCK (pad);
-
   return res;
 }
 
-static GstIterator *
-gst_rtp_ssrc_demux_iterate_internal_links_sink (GstPad * pad)
+static GList *
+gst_rtp_ssrc_demux_internal_links (GstPad * pad)
 {
   GstRtpSsrcDemux *demux;
-  GstIterator *it = NULL;
-  const gchar *prefix = NULL;
+  GList *res = NULL;
+  GSList *walk;
 
   demux = GST_RTP_SSRC_DEMUX (gst_pad_get_parent (pad));
 
-  if (!demux)
-    return NULL;
+  GST_PAD_LOCK (demux);
+  for (walk = demux->srcpads; walk; walk = g_slist_next (walk)) {
+    GstRtpSsrcDemuxPad *dpad = (GstRtpSsrcDemuxPad *) walk->data;
 
-  if (pad == demux->rtp_sink)
-    prefix = "src_";
-  else if (pad == demux->rtcp_sink)
-    prefix = "rtcp_src_";
-  else
-    g_assert_not_reached ();
-
-  it = gst_element_iterate_src_pads (GST_ELEMENT (demux));
-
-  it = gst_iterator_filter (it, src_pad_compare_func, (gpointer) prefix);
+    if (pad == demux->rtp_sink) {
+      res = g_list_prepend (res, dpad->rtp_pad);
+    } else if (pad == demux->rtcp_sink) {
+      res = g_list_prepend (res, dpad->rtcp_pad);
+    } else if (pad == dpad->rtp_pad) {
+      res = g_list_prepend (res, demux->rtp_sink);
+      break;
+    } else if (pad == dpad->rtcp_pad) {
+      res = g_list_prepend (res, demux->rtcp_sink);
+      break;
+    }
+  }
+  GST_PAD_UNLOCK (demux);
 
   gst_object_unref (demux);
-  return it;
+  return res;
 }
 
-
 static gboolean
 gst_rtp_ssrc_demux_src_query (GstPad * pad, GstQuery * query)
 {
@@ -756,8 +657,6 @@ gst_rtp_ssrc_demux_src_query (GstPad * pad, GstQuery * query)
   gboolean res = FALSE;
 
   demux = GST_RTP_SSRC_DEMUX (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (demux == NULL))
-    return FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_LATENCY:
