@@ -23,7 +23,6 @@
 #include <gst/gst.h>
 #include <gst/video/video.h>
 #include "videomixerpad.h"
-#include "blend.h"
 
 GST_DEBUG_CATEGORY_EXTERN (gst_videomixer_debug);
 #define GST_CAT_DEFAULT gst_videomixer_debug
@@ -82,7 +81,6 @@ struct _GstVideoMixer
   gint numpads;
 
   GstClockTime last_ts;
-  GstClockTime last_duration;
 
   /* the master pad */
   GstVideoMixerPad *master;
@@ -98,26 +96,20 @@ struct _GstVideoMixer
 
   gint fps_n;
   gint fps_d;
-
+  
   /* Next available sinkpad index */
   gint next_sinkpad;
 
   /* sink event handling */
   GstPadEventFunction collect_event;
   guint64	segment_position;
+  gdouble	segment_rate;
 
-  /* Current downstream segment */
-  GstSegment    segment;
+  void (*blend) (guint8 * src, gint xpos, gint ypos, gint src_width, gint src_height, gdouble src_alpha,
+                 guint8 * dest, gint dest_width, gint dest_height);
+  void (*fill_checker) (guint8 * dest, gint width, gint height);
 
-  /* QoS stuff */
-  gdouble proportion;
-  GstClockTime earliest_time;
-
-  BlendFunction blend;
-  FillCheckerFunction fill_checker;
-  FillColorFunction fill_color;
-
-  gboolean flush_stop_pending;
+  void (*fill_color) (guint8 * dest, gint width, gint height, gint colY, gint colU, gint colV);
 };
 
 struct _GstVideoMixerClass
